@@ -1,4 +1,3 @@
-use crate::context::consts;
 use crate::core::*;
 
 ///
@@ -20,7 +19,11 @@ impl UniformBuffer {
     /// The variables are initialized to 0.
     ///
     pub fn new(context: &Context, sizes: &[u32]) -> ThreeDResult<UniformBuffer> {
-        let id = context.create_buffer().unwrap();
+        let id = unsafe {
+            context
+                .create_buffer()
+                .map_err(|e| CoreError::BufferCreation(e))?
+        };
 
         let mut offsets = Vec::new();
         let mut length = 0;
@@ -35,12 +38,15 @@ impl UniformBuffer {
             data: vec![0.0; length as usize],
         };
         buffer.send();
+        context.error_check()?;
         Ok(buffer)
     }
 
     pub(crate) fn bind(&self, id: u32) {
-        self.context
-            .bind_buffer_base(consts::UNIFORM_BUFFER, id, &self.id);
+        unsafe {
+            self.context
+                .bind_buffer_base(crate::context::UNIFORM_BUFFER, id, Some(self.id))
+        };
     }
 
     ///
@@ -63,7 +69,7 @@ impl UniformBuffer {
             .splice(offset..offset + length, data.iter().cloned());
         self.send();
         //TODO: Send to GPU (contextBufferSubData)
-        Ok(())
+        self.context.error_check()
     }
 
     ///
@@ -91,15 +97,24 @@ impl UniformBuffer {
     }
 
     fn send(&self) {
-        self.context.bind_buffer(consts::UNIFORM_BUFFER, &self.id);
-        self.context
-            .buffer_data_f32(consts::UNIFORM_BUFFER, &self.data, consts::STATIC_DRAW);
-        self.context.unbind_buffer(consts::UNIFORM_BUFFER);
+        unsafe {
+            self.context
+                .bind_buffer(crate::context::UNIFORM_BUFFER, Some(self.id));
+            self.context.buffer_data_u8_slice(
+                crate::context::UNIFORM_BUFFER,
+                to_byte_slice(&self.data),
+                crate::context::STATIC_DRAW,
+            );
+            self.context
+                .bind_buffer(crate::context::UNIFORM_BUFFER, None);
+        }
     }
 }
 
 impl Drop for UniformBuffer {
     fn drop(&mut self) {
-        self.context.delete_buffer(&self.id);
+        unsafe {
+            self.context.delete_buffer(self.id);
+        }
     }
 }
