@@ -3,7 +3,7 @@ use crate::core::*;
 ///
 /// An array of indices. Supports different data types.
 ///
-#[derive(Debug)]
+#[derive(Clone)]
 pub enum Indices {
     /// Uses unsigned 8 bit integer for each index.
     U8(Vec<u8>),
@@ -15,50 +15,175 @@ pub enum Indices {
 
 impl Indices {
     ///
-    /// Returns all the indices as an `u32` data type. Clones all of the indices, so do not use it too often.
+    /// Converts all the indices as `u32` data type.
     ///
-    pub fn into_u32(&self) -> Vec<u32> {
+    pub fn into_u32(self) -> Vec<u32> {
         match self {
-            Self::U8(ind) => ind.iter().map(|i| *i as u32).collect::<Vec<u32>>(),
-            Self::U16(ind) => ind.iter().map(|i| *i as u32).collect::<Vec<u32>>(),
-            Self::U32(ind) => ind.clone(),
+            Self::U8(mut values) => values.drain(..).map(|i| i as u32).collect::<Vec<u32>>(),
+            Self::U16(mut values) => values.drain(..).map(|i| i as u32).collect::<Vec<u32>>(),
+            Self::U32(values) => values,
+        }
+    }
+
+    ///
+    /// Clones and converts all the indices as `u32` data type.
+    ///
+    pub fn to_u32(&self) -> Vec<u32> {
+        match self {
+            Self::U8(values) => values.iter().map(|i| *i as u32).collect::<Vec<u32>>(),
+            Self::U16(values) => values.iter().map(|i| *i as u32).collect::<Vec<u32>>(),
+            Self::U32(values) => values.clone(),
+        }
+    }
+
+    ///
+    /// Returns the number of indices.
+    ///
+    pub fn len(&self) -> usize {
+        match self {
+            Self::U8(values) => values.len(),
+            Self::U16(values) => values.len(),
+            Self::U32(values) => values.len(),
         }
     }
 }
 
+impl std::default::Default for Indices {
+    fn default() -> Self {
+        Self::U32(Vec::new())
+    }
+}
+
+impl std::fmt::Debug for Indices {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut d = f.debug_struct("Indices");
+        match self {
+            Self::U8(ind) => d.field("u8", &ind.len()),
+            Self::U16(ind) => d.field("u16", &ind.len()),
+            Self::U32(ind) => d.field("u32", &ind.len()),
+        };
+        d.finish()
+    }
+}
+
+///
+/// An array of positions. Supports f32 and f64 data types.
+///
+#[derive(Clone)]
+pub enum Positions {
+    /// Uses 32 bit float for the vertex positions.
+    F32(Vec<Vector3<f32>>),
+    /// Uses 64 bit float for the vertex positions.
+    F64(Vec<Vector3<f64>>),
+}
+
+impl Positions {
+    ///
+    /// Converts and returns all the indices as `f32` data type.
+    ///
+    pub fn into_f32(self) -> Vec<Vec3> {
+        match self {
+            Self::F32(values) => values,
+            Self::F64(mut values) => values
+                .drain(..)
+                .map(|v| vec3(v.x as f32, v.y as f32, v.z as f32))
+                .collect::<Vec<_>>(),
+        }
+    }
+
+    ///
+    /// Clones and converts all the positions as `f32` data type.
+    ///
+    pub fn to_f32(&self) -> Vec<Vec3> {
+        match self {
+            Self::F32(values) => values.clone(),
+            Self::F64(values) => values
+                .iter()
+                .map(|v| vec3(v.x as f32, v.y as f32, v.z as f32))
+                .collect::<Vec<_>>(),
+        }
+    }
+
+    ///
+    /// Returns the number of positions.
+    ///
+    pub fn len(&self) -> usize {
+        match self {
+            Self::F32(values) => values.len(),
+            Self::F64(values) => values.len(),
+        }
+    }
+}
+
+impl std::default::Default for Positions {
+    fn default() -> Self {
+        Self::F32(Vec::new())
+    }
+}
+
+impl std::fmt::Debug for Positions {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut d = f.debug_struct("Positions");
+        match self {
+            Self::F32(ind) => d.field("f32", &ind.len()),
+            Self::F64(ind) => d.field("f64", &ind.len()),
+        };
+        d.finish()
+    }
+}
+
+/// See [CpuMesh]
+#[deprecated = "Renamed to CpuMesh"]
+pub type CPUMesh = CpuMesh;
+
 ///
 /// A CPU-side version of a triangle mesh.
 /// Can be constructed manually or loaded via [io](crate::io)
 /// or via the utility functions for generating simple triangle meshes.
 ///
-#[derive(Default, Debug)]
-pub struct CPUMesh {
+#[derive(Default)]
+pub struct CpuMesh {
     /// Name.
     pub name: String,
-    /// Name of the associated material, use this to match with [CPUMaterial::name].
+    /// Name of the associated material, use this to match with [CpuMaterial::name].
     pub material_name: Option<String>,
-    /// The positions of the vertices. Three contiguous floats defines a 3D position `(x, y, z)`, therefore the length must be divisable by 3.
-    /// If there is no indices associated with this mesh, three contiguous positions defines a triangle, in that case, the length must also be divisable by 9.
-    pub positions: Vec<f32>,
+    /// The positions of the vertices.
+    /// If there is no indices associated with this mesh, three contiguous positions defines a triangle, in that case, the length must be divisable by 3.
+    pub positions: Positions,
     /// The indices into the positions, normals, uvs and colors arrays which defines the three vertices of a triangle. Three contiguous indices defines a triangle, therefore the length must be divisable by 3.
     pub indices: Option<Indices>,
-    /// The normals of the vertices. Three contiguous floats defines a normal `(x, y, z)`, therefore the length must be divisable by 3.
-    pub normals: Option<Vec<f32>>,
+    /// The normals of the vertices.
+    pub normals: Option<Vec<Vec3>>,
     /// The tangents of the vertices, orthogonal direction to the normal.
-    /// Three contiguous floats defines a tangent `(x, y, z)` and a value that specifies the handedness (either -1.0 or 1.0), therefore the length must be divisable by 4.
-    pub tangents: Option<Vec<f32>>,
-    /// The uv coordinates of the vertices. Two contiguous floats defines a coordinate `(u, v)`, therefore the length must be divisable by 2.
-    pub uvs: Option<Vec<f32>>,
-    /// The colors of the vertices. Four contiguous bytes defines a color `(r, g, b, a)`, therefore the length must be divisable by 4.
+    /// The fourth value specifies the handedness (either -1.0 or 1.0).
+    pub tangents: Option<Vec<Vec4>>,
+    /// The uv coordinates of the vertices.
+    pub uvs: Option<Vec<Vec2>>,
+    /// The colors of the vertices.
     /// The colors are assumed to be in linear space.
-    pub colors: Option<Vec<u8>>,
+    pub colors: Option<Vec<Color>>,
 }
 
-impl CPUMesh {
+impl std::fmt::Debug for CpuMesh {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        let mut d = f.debug_struct("CpuMesh");
+        d.field("name", &self.name);
+        d.field("material name", &self.material_name);
+        d.field("positions", &self.positions.len());
+        d.field("indices", &self.indices);
+        d.field("normals", &self.normals.as_ref().map(|v| v.len()));
+        d.field("tangents", &self.tangents.as_ref().map(|v| v.len()));
+        d.field("uvs", &self.uvs.as_ref().map(|v| v.len()));
+        d.field("colors", &self.colors.as_ref().map(|v| v.len()));
+        d.finish()
+    }
+}
+
+impl CpuMesh {
     ///
     /// Returns the material for this mesh in the given list of materials. Returns `None` if no suitable material can be found.
     ///
-    pub fn material<'a>(&self, materials: &'a [CPUMaterial]) -> Option<&'a CPUMaterial> {
+    pub fn material<'a>(&self, materials: &'a [CpuMaterial]) -> Option<&'a CpuMaterial> {
         self.material_name.as_ref().and_then(|name| {
             materials
                 .iter()
@@ -70,46 +195,41 @@ impl CPUMesh {
     ///
     /// Transforms the mesh by the given transformation.
     ///
-    pub fn transform(&mut self, transform: &Mat4) {
-        for i in 0..self.positions.len() / 3 {
-            let p = (transform
-                * vec4(
-                    self.positions[i * 3],
-                    self.positions[i * 3 + 1],
-                    self.positions[i * 3 + 2],
-                    1.0,
-                ))
-            .truncate();
-            self.positions[i * 3] = p.x;
-            self.positions[i * 3 + 1] = p.y;
-            self.positions[i * 3 + 2] = p.z;
-        }
-        let normal_transform = transform.invert().unwrap().transpose();
+    pub fn transform(&mut self, transform: &Mat4) -> ThreeDResult<()> {
+        match self.positions {
+            Positions::F32(ref mut positions) => {
+                for pos in positions.iter_mut() {
+                    *pos = (transform * pos.extend(1.0)).truncate();
+                }
+            }
+            Positions::F64(ref mut positions) => {
+                let t = transform.cast::<f64>().unwrap();
+                for pos in positions.iter_mut() {
+                    *pos = (t * pos.extend(1.0)).truncate();
+                }
+            }
+        };
 
-        if let Some(ref mut normals) = self.normals {
-            for i in 0..normals.len() / 3 {
-                let n = normal_transform
-                    * vec4(normals[i * 3], normals[i * 3 + 1], normals[i * 3 + 2], 1.0);
-                normals[i * 3] = n.x;
-                normals[i * 3 + 1] = n.y;
-                normals[i * 3 + 2] = n.z;
-            }
-        }
+        if self.normals.is_some() || self.tangents.is_some() {
+            let normal_transform = transform
+                .invert()
+                .ok_or(CoreError::FailedInvertingTransformationMatrix)?
+                .transpose();
 
-        if let Some(ref mut tangents) = self.tangents {
-            for i in 0..tangents.len() / 4 {
-                let t = normal_transform
-                    * vec4(
-                        tangents[i * 4],
-                        tangents[i * 4 + 1],
-                        tangents[i * 4 + 2],
-                        1.0,
-                    );
-                tangents[i * 4] = t.x;
-                tangents[i * 4 + 1] = t.y;
-                tangents[i * 4 + 2] = t.z;
+            if let Some(ref mut normals) = self.normals {
+                for n in normals.iter_mut() {
+                    *n = (normal_transform * n.extend(1.0)).truncate();
+                }
+            }
+            if let Some(ref mut tangents) = self.tangents {
+                for t in tangents.iter_mut() {
+                    *t = (normal_transform * t.truncate().extend(1.0))
+                        .truncate()
+                        .extend(t.w);
+                }
             }
         }
+        Ok(())
     }
 
     ///
@@ -119,18 +239,33 @@ impl CPUMesh {
         let indices = vec![0u8, 1, 2, 2, 3, 0];
         let halfsize = 1.0;
         let positions = vec![
-            -halfsize, -halfsize, 0.0, halfsize, -halfsize, 0.0, halfsize, halfsize, 0.0,
-            -halfsize, halfsize, 0.0,
+            vec3(-halfsize, -halfsize, 0.0),
+            vec3(halfsize, -halfsize, 0.0),
+            vec3(halfsize, halfsize, 0.0),
+            vec3(-halfsize, halfsize, 0.0),
+        ];
+        let normals = vec![
+            vec3(0.0, 0.0, 1.0),
+            vec3(0.0, 0.0, 1.0),
+            vec3(0.0, 0.0, 1.0),
+            vec3(0.0, 0.0, 1.0),
         ];
-        let normals = vec![0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0];
         let tangents = vec![
-            1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0,
+            vec4(1.0, 0.0, 0.0, 1.0),
+            vec4(1.0, 0.0, 0.0, 1.0),
+            vec4(1.0, 0.0, 0.0, 1.0),
+            vec4(1.0, 0.0, 0.0, 1.0),
         ];
-        let uvs = vec![0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 1.0];
-        CPUMesh {
+        let uvs = vec![
+            vec2(0.0, 1.0),
+            vec2(1.0, 1.0),
+            vec2(1.0, 0.0),
+            vec2(0.0, 0.0),
+        ];
+        CpuMesh {
             name: "square".to_string(),
             indices: Some(Indices::U8(indices)),
-            positions,
+            positions: Positions::F32(positions),
             normals: Some(normals),
             tangents: Some(tangents),
             uvs: Some(uvs),
@@ -148,13 +283,8 @@ impl CPUMesh {
         for j in 0..angle_subdivisions {
             let angle = 2.0 * std::f32::consts::PI * j as f32 / angle_subdivisions as f32;
 
-            positions.push(angle.cos());
-            positions.push(angle.sin());
-            positions.push(0.0);
-
-            normals.push(0.0);
-            normals.push(0.0);
-            normals.push(1.0);
+            positions.push(vec3(angle.cos(), angle.sin(), 0.0));
+            normals.push(vec3(0.0, 0.0, 1.0));
         }
 
         for j in 0..angle_subdivisions {
@@ -162,10 +292,10 @@ impl CPUMesh {
             indices.push(j as u16);
             indices.push(((j + 1) % angle_subdivisions) as u16);
         }
-        CPUMesh {
+        CpuMesh {
             name: "circle".to_string(),
             indices: Some(Indices::U16(indices)),
-            positions,
+            positions: Positions::F32(positions),
             normals: Some(normals),
             ..Default::default()
         }
@@ -179,13 +309,8 @@ impl CPUMesh {
         let mut indices = Vec::new();
         let mut normals = Vec::new();
 
-        positions.push(0.0);
-        positions.push(0.0);
-        positions.push(1.0);
-
-        normals.push(0.0);
-        normals.push(0.0);
-        normals.push(1.0);
+        positions.push(vec3(0.0, 0.0, 1.0));
+        normals.push(vec3(0.0, 0.0, 1.0));
 
         for j in 0..angle_subdivisions * 2 {
             let j1 = (j + 1) % (angle_subdivisions * 2);
@@ -206,13 +331,8 @@ impl CPUMesh {
                 let x = sin_theta * phi.cos();
                 let y = sin_theta * phi.sin();
                 let z = cos_theta;
-                positions.push(x);
-                positions.push(y);
-                positions.push(z);
-
-                normals.push(x);
-                normals.push(y);
-                normals.push(z);
+                positions.push(vec3(x, y, z));
+                normals.push(vec3(x, y, z));
 
                 if i != angle_subdivisions - 2 {
                     let j1 = (j + 1) % (angle_subdivisions * 2);
@@ -225,13 +345,8 @@ impl CPUMesh {
                 }
             }
         }
-        positions.push(0.0);
-        positions.push(0.0);
-        positions.push(-1.0);
-
-        normals.push(0.0);
-        normals.push(0.0);
-        normals.push(-1.0);
+        positions.push(vec3(0.0, 0.0, -1.0));
+        normals.push(vec3(0.0, 0.0, -1.0));
 
         let i = 1 + (angle_subdivisions - 2) * angle_subdivisions * 2;
         for j in 0..angle_subdivisions * 2 {
@@ -241,10 +356,10 @@ impl CPUMesh {
             indices.push((i + j1) as u16);
         }
 
-        CPUMesh {
+        CpuMesh {
             name: "sphere".to_string(),
             indices: Some(Indices::U16(indices)),
-            positions,
+            positions: Positions::F32(positions),
             normals: Some(normals),
             ..Default::default()
         }
@@ -255,23 +370,83 @@ impl CPUMesh {
     ///
     pub fn cube() -> Self {
         let positions = vec![
-            1.0, 1.0, -1.0, -1.0, 1.0, -1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0,
-            1.0, -1.0, -1.0, -1.0, -1.0, 1.0, -1.0, -1.0, 1.0, -1.0, 1.0, 1.0, -1.0, 1.0, -1.0,
-            -1.0, 1.0, -1.0, -1.0, -1.0, 1.0, -1.0, -1.0, -1.0, -1.0, -1.0, 1.0, 1.0, -1.0, -1.0,
-            1.0, -1.0, 1.0, 1.0, -1.0, -1.0, -1.0, -1.0, -1.0, -1.0, 1.0, 1.0, -1.0, 1.0, 1.0, 1.0,
-            1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 1.0, -1.0, -1.0, 1.0, 1.0, -1.0, -1.0, 1.0, 1.0, -1.0,
-            1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, -1.0, 1.0, 1.0, -1.0, -1.0, -1.0, 1.0, -1.0, -1.0,
-            -1.0, -1.0, -1.0, 1.0, 1.0, -1.0, -1.0, 1.0, -1.0, 1.0, 1.0, -1.0, -1.0, -1.0,
+            vec3(1.0, 1.0, -1.0),
+            vec3(-1.0, 1.0, -1.0),
+            vec3(1.0, 1.0, 1.0),
+            vec3(-1.0, 1.0, 1.0),
+            vec3(1.0, 1.0, 1.0),
+            vec3(-1.0, 1.0, -1.0),
+            vec3(-1.0, -1.0, -1.0),
+            vec3(1.0, -1.0, -1.0),
+            vec3(1.0, -1.0, 1.0),
+            vec3(1.0, -1.0, 1.0),
+            vec3(-1.0, -1.0, 1.0),
+            vec3(-1.0, -1.0, -1.0),
+            vec3(1.0, -1.0, -1.0),
+            vec3(-1.0, -1.0, -1.0),
+            vec3(1.0, 1.0, -1.0),
+            vec3(-1.0, 1.0, -1.0),
+            vec3(1.0, 1.0, -1.0),
+            vec3(-1.0, -1.0, -1.0),
+            vec3(-1.0, -1.0, 1.0),
+            vec3(1.0, -1.0, 1.0),
+            vec3(1.0, 1.0, 1.0),
+            vec3(1.0, 1.0, 1.0),
+            vec3(-1.0, 1.0, 1.0),
+            vec3(-1.0, -1.0, 1.0),
+            vec3(1.0, -1.0, -1.0),
+            vec3(1.0, 1.0, -1.0),
+            vec3(1.0, 1.0, 1.0),
+            vec3(1.0, 1.0, 1.0),
+            vec3(1.0, -1.0, 1.0),
+            vec3(1.0, -1.0, -1.0),
+            vec3(-1.0, 1.0, -1.0),
+            vec3(-1.0, -1.0, -1.0),
+            vec3(-1.0, 1.0, 1.0),
+            vec3(-1.0, -1.0, 1.0),
+            vec3(-1.0, 1.0, 1.0),
+            vec3(-1.0, -1.0, -1.0),
         ];
         let uvs = vec![
-            1.0, 0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 1.0, 1.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0,
-            1.0, 1.0, 1.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 1.0, 1.0, 1.0,
-            0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 1.0, 1.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0,
-            0.0, 1.0, 1.0, 1.0, 1.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 1.0, 0.0, 1.0,
-            1.0, 1.0, 0.0, 0.0,
+            vec2(1.0, 0.0),
+            vec2(0.0, 0.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 1.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 0.0),
+            vec2(0.0, 0.0),
+            vec2(1.0, 0.0),
+            vec2(1.0, 1.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 1.0),
+            vec2(0.0, 0.0),
+            vec2(1.0, 0.0),
+            vec2(0.0, 0.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 1.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 0.0),
+            vec2(0.0, 0.0),
+            vec2(1.0, 0.0),
+            vec2(1.0, 1.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 1.0),
+            vec2(0.0, 0.0),
+            vec2(0.0, 0.0),
+            vec2(1.0, 0.0),
+            vec2(1.0, 1.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 1.0),
+            vec2(0.0, 0.0),
+            vec2(1.0, 0.0),
+            vec2(0.0, 0.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 1.0),
+            vec2(1.0, 1.0),
+            vec2(0.0, 0.0),
         ];
-        let mut mesh = CPUMesh {
-            positions,
+        let mut mesh = CpuMesh {
+            positions: Positions::F32(positions),
             uvs: Some(uvs),
             ..Default::default()
         };
@@ -292,9 +467,7 @@ impl CPUMesh {
             for j in 0..angle_subdivisions {
                 let angle = 2.0 * std::f32::consts::PI * j as f32 / angle_subdivisions as f32;
 
-                positions.push(x);
-                positions.push(angle.cos());
-                positions.push(angle.sin());
+                positions.push(vec3(x, angle.cos(), angle.sin()));
             }
         }
         for i in 0..length_subdivisions {
@@ -310,7 +483,7 @@ impl CPUMesh {
         }
         let mut mesh = Self {
             name: "cylinder".to_string(),
-            positions,
+            positions: Positions::F32(positions),
             indices: Some(Indices::U16(indices)),
             ..Default::default()
         };
@@ -330,9 +503,7 @@ impl CPUMesh {
             for j in 0..angle_subdivisions {
                 let angle = 2.0 * std::f32::consts::PI * j as f32 / angle_subdivisions as f32;
 
-                positions.push(x);
-                positions.push(angle.cos() * (1.0 - x));
-                positions.push(angle.sin() * (1.0 - x));
+                positions.push(vec3(x, angle.cos() * (1.0 - x), angle.sin() * (1.0 - x)));
             }
         }
         for i in 0..length_subdivisions {
@@ -348,7 +519,7 @@ impl CPUMesh {
         }
         let mut mesh = Self {
             name: "cone".to_string(),
-            positions,
+            positions: Positions::F32(positions),
             indices: Some(Indices::U16(indices)),
             ..Default::default()
         };
@@ -362,24 +533,31 @@ impl CPUMesh {
     ///
     pub fn arrow(tail_length: f32, tail_radius: f32, angle_subdivisions: u32) -> Self {
         let mut arrow = Self::cylinder(angle_subdivisions);
-        arrow.transform(&Mat4::from_nonuniform_scale(
-            tail_length,
-            tail_radius,
-            tail_radius,
-        ));
+        arrow
+            .transform(&Mat4::from_nonuniform_scale(
+                tail_length,
+                tail_radius,
+                tail_radius,
+            ))
+            .unwrap();
         arrow.name = "arrow".to_string();
         let mut cone = Self::cone(angle_subdivisions);
         cone.transform(
             &(Mat4::from_translation(vec3(tail_length, 0.0, 0.0))
                 * Mat4::from_nonuniform_scale(1.0 - tail_length, 1.0, 1.0)),
-        );
+        )
+        .unwrap();
         let mut indices = arrow.indices.unwrap().into_u32();
         let cone_indices = cone.indices.unwrap().into_u32();
         let offset = indices.iter().max().unwrap() + 1;
         indices.extend(cone_indices.iter().map(|i| i + offset));
         arrow.indices = Some(Indices::U16(indices.iter().map(|i| *i as u16).collect()));
 
-        arrow.positions.extend(cone.positions);
+        if let Positions::F32(ref mut p) = arrow.positions {
+            if let Positions::F32(ref p2) = cone.positions {
+                p.extend(p2);
+            }
+        }
         arrow
             .normals
             .as_mut()
@@ -393,29 +571,31 @@ impl CPUMesh {
     /// It will override the current normals if they already exist.
     ///
     pub fn compute_normals(&mut self) {
-        let mut normals = vec![0.0f32; self.positions.len()];
+        let mut normals = vec![vec3(0.0, 0.0, 0.0); self.positions.len()];
         self.for_each_triangle(|i0, i1, i2| {
-            let p0 = self.position(i0);
-            let p1 = self.position(i1);
-            let p2 = self.position(i2);
-            let normal = (p1 - p0).cross(p2 - p0);
-            normals[i0 * 3] += normal.x;
-            normals[i0 * 3 + 1] += normal.y;
-            normals[i0 * 3 + 2] += normal.z;
-            normals[i1 * 3] += normal.x;
-            normals[i1 * 3 + 1] += normal.y;
-            normals[i1 * 3 + 2] += normal.z;
-            normals[i2 * 3] += normal.x;
-            normals[i2 * 3 + 1] += normal.y;
-            normals[i2 * 3 + 2] += normal.z;
+            let normal = match self.positions {
+                Positions::F32(ref positions) => {
+                    let p0 = positions[i0];
+                    let p1 = positions[i1];
+                    let p2 = positions[i2];
+                    (p1 - p0).cross(p2 - p0)
+                }
+                Positions::F64(ref positions) => {
+                    let p0 = positions[i0];
+                    let p1 = positions[i1];
+                    let p2 = positions[i2];
+                    let n = (p1 - p0).cross(p2 - p0);
+                    vec3(n.x as f32, n.y as f32, n.z as f32)
+                }
+            };
+            normals[i0] += normal;
+            normals[i1] += normal;
+            normals[i2] += normal;
         });
 
-        self.for_each_vertex(|i| {
-            let normal = vec3(normals[3 * i], normals[3 * i + 1], normals[3 * i + 2]).normalize();
-            normals[3 * i] = normal.x;
-            normals[3 * i + 1] = normal.y;
-            normals[3 * i + 2] = normal.z;
-        });
+        for n in normals.iter_mut() {
+            *n = n.normalize();
+        }
         self.normals = Some(normals);
     }
 
@@ -427,16 +607,24 @@ impl CPUMesh {
         if self.normals.is_none() || self.uvs.is_none() {
             Err(CoreError::FailedComputingTangents)?;
         }
-        let mut tan1 = vec![vec3(0.0, 0.0, 0.0); self.positions.len() / 3];
-        let mut tan2 = vec![vec3(0.0, 0.0, 0.0); self.positions.len() / 3];
+        let mut tan1 = vec![vec3(0.0, 0.0, 0.0); self.positions.len()];
+        let mut tan2 = vec![vec3(0.0, 0.0, 0.0); self.positions.len()];
 
         self.for_each_triangle(|i0, i1, i2| {
-            let a = self.position(i0);
-            let b = self.position(i1);
-            let c = self.position(i2);
-            let uva = self.uv(i0).unwrap();
-            let uvb = self.uv(i1).unwrap();
-            let uvc = self.uv(i2).unwrap();
+            let (a, b, c) = match self.positions {
+                Positions::F32(ref positions) => (positions[i0], positions[i1], positions[i2]),
+                Positions::F64(ref positions) => {
+                    let (a, b, c) = (positions[i0], positions[i1], positions[i2]);
+                    (
+                        vec3(a.x as f32, a.y as f32, a.z as f32),
+                        vec3(b.x as f32, b.y as f32, b.z as f32),
+                        vec3(c.x as f32, c.y as f32, c.z as f32),
+                    )
+                }
+            };
+            let uva = self.uvs.as_ref().unwrap()[i0];
+            let uvb = self.uvs.as_ref().unwrap()[i1];
+            let uvc = self.uvs.as_ref().unwrap()[i2];
 
             let ba = b - a;
             let ca = c - a;
@@ -458,9 +646,9 @@ impl CPUMesh {
             }
         });
 
-        let mut tangents = vec![0.0f32; 4 * self.positions.len() / 3];
+        let mut tangents = vec![vec4(0.0, 0.0, 0.0, 0.0); self.positions.len()];
         self.for_each_vertex(|index| {
-            let normal = self.normal(index).unwrap();
+            let normal = self.normals.as_ref().unwrap()[index];
             let t = tan1[index];
             let tangent = (t - normal * normal.dot(t)).normalize();
             let handedness = if normal.cross(tangent).dot(tan2[index]) < 0.0 {
@@ -468,10 +656,7 @@ impl CPUMesh {
             } else {
                 -1.0
             };
-            tangents[index * 4] = tangent.x;
-            tangents[index * 4 + 1] = tangent.y;
-            tangents[index * 4 + 2] = tangent.z;
-            tangents[index * 4 + 3] = handedness;
+            tangents[index] = tangent.extend(handedness);
         });
 
         self.tangents = Some(tangents);
@@ -482,7 +667,7 @@ impl CPUMesh {
     ///  Iterates over all vertices in this mesh and calls the callback function with the index for each vertex.
     ///
     pub fn for_each_vertex(&self, mut callback: impl FnMut(usize)) {
-        for i in 0..self.positions.len() / 3 {
+        for i in 0..self.positions.len() {
             callback(i);
         }
     }
@@ -517,123 +702,72 @@ impl CPUMesh {
                 }
             }
             None => {
-                for face in 0..self.positions.len() / 9 {
+                for face in 0..self.positions.len() / 3 {
                     callback(face * 3, face * 3 + 1, face * 3 + 2);
                 }
             }
         }
     }
 
-    ///
-    /// Returns the position of the vertex with the given index.
-    ///
-    pub fn position(&self, vertex_index: usize) -> Vec3 {
-        vec3(
-            self.positions[3 * vertex_index],
-            self.positions[3 * vertex_index + 1],
-            self.positions[3 * vertex_index + 2],
-        )
-    }
-
-    ///
-    /// Returns the normal of the vertex with the given index.
-    ///
-    pub fn normal(&self, vertex_index: usize) -> Option<Vec3> {
-        self.normals.as_ref().map(|normals| {
-            vec3(
-                normals[3 * vertex_index],
-                normals[3 * vertex_index + 1],
-                normals[3 * vertex_index + 2],
-            )
-        })
-    }
-
-    ///
-    /// Returns the uv coordinates of the vertex with the given index.
-    ///
-    pub fn uv(&self, vertex_index: usize) -> Option<Vec2> {
-        self.uvs
-            .as_ref()
-            .map(|uvs| vec2(uvs[2 * vertex_index], uvs[2 * vertex_index + 1]))
-    }
-
     ///
     /// Computes the axis aligned bounding box of the mesh.
     ///
     pub fn compute_aabb(&self) -> AxisAlignedBoundingBox {
-        AxisAlignedBoundingBox::new_with_positions(&self.positions)
+        match self.positions {
+            Positions::F32(ref positions) => AxisAlignedBoundingBox::new_with_positions(positions),
+            Positions::F64(ref positions) => AxisAlignedBoundingBox::new_with_positions(
+                &positions
+                    .iter()
+                    .map(|v| vec3(v.x as f32, v.y as f32, v.z as f32))
+                    .collect::<Vec<_>>(),
+            ),
+        }
     }
 
     ///
     /// Returns an error if the mesh is not valid.
     ///
     pub fn validate(&self) -> ThreeDResult<()> {
-        if let Some(ref indices) = self.indices {
+        let vertex_count = if let Some(ref indices) = self.indices {
             let index_count = match indices {
                 Indices::U8(ind) => ind.len(),
                 Indices::U16(ind) => ind.len(),
                 Indices::U32(ind) => ind.len(),
             };
             if index_count % 3 != 0 {
-                Err(CoreError::InvalidBufferLength(
-                    "index".to_string(),
-                    index_count,
-                ))?;
+                Err(CoreError::InvalidNumberOfVertices(index_count))?;
             }
-            if self.positions.len() % 3 != 0 {
-                Err(CoreError::InvalidBufferLength(
-                    "position".to_string(),
-                    index_count,
-                ))?;
-            }
-            if let Some(ref data) = self.normals {
-                if data.len() % 3 != 0 {
-                    Err(CoreError::InvalidBufferLength(
-                        "normal".to_string(),
-                        index_count,
-                    ))?;
-                }
-            }
-            if let Some(ref data) = self.colors {
-                if data.len() % 4 != 0 {
-                    Err(CoreError::InvalidBufferLength(
-                        "color".to_string(),
-                        index_count,
-                    ))?;
-                }
-            }
-            if let Some(ref data) = self.uvs {
-                if data.len() % 2 != 0 {
-                    Err(CoreError::InvalidBufferLength(
-                        "uv coordinate".to_string(),
-                        index_count,
-                    ))?;
-                }
-            }
-            if cfg!(debug) {
-                let indices_valid = match indices {
-                    Indices::U8(ind) => {
-                        let len = self.positions.len();
-                        ind.iter().all(|&i| (i as usize) < len)
-                    }
-                    Indices::U16(ind) => {
-                        let len = self.positions.len();
-                        ind.iter().all(|&i| (i as usize) < len)
-                    }
-                    Indices::U32(ind) => {
-                        let len = self.positions.len();
-                        ind.iter().all(|&i| (i as usize) < len)
-                    }
-                };
-                if !indices_valid {
-                    Err(CoreError::InvalidIndexBuffer(self.positions.len()))?;
-                }
+            match indices {
+                Indices::U8(ind) => ind.iter().max().map(|m| m + 1).unwrap_or(0) as usize,
+                Indices::U16(ind) => ind.iter().max().map(|m| m + 1).unwrap_or(0) as usize,
+                Indices::U32(ind) => ind.iter().max().map(|m| m + 1).unwrap_or(0) as usize,
             }
         } else {
-            if self.positions.len() % 9 != 0 {
-                Err(CoreError::InvalidPositionBuffer(self.positions.len()))?;
+            let vertex_count = self.positions.len();
+            if vertex_count % 3 != 0 {
+                Err(CoreError::InvalidNumberOfVertices(vertex_count))?;
             }
+            vertex_count
         };
+        let buffer_check = |length: Option<usize>, name: &str| -> ThreeDResult<()> {
+            if let Some(length) = length {
+                if length < vertex_count {
+                    Err(CoreError::InvalidBufferLength(
+                        name.to_string(),
+                        vertex_count,
+                        length,
+                    ))?;
+                }
+            }
+            Ok(())
+        };
+
+        buffer_check(Some(self.positions.len()), "position")?;
+        buffer_check(self.normals.as_ref().map(|b| b.len()), "normal")?;
+        buffer_check(self.tangents.as_ref().map(|b| b.len()), "tangent")?;
+        buffer_check(self.colors.as_ref().map(|b| b.len()), "color")?;
+        buffer_check(self.uvs.as_ref().map(|b| b.len()), "uv coordinate")?;
+
         Ok(())
     }
 }
