@@ -1,4 +1,3 @@
-#![warn(missing_docs)]
 //!
 //! Contains functionality to load any type of asset runtime on both desktop and web as well as parsers for different image and 3D model formats.
 //! Also includes functionality to save data which is limited to desktop.
@@ -43,9 +42,17 @@ pub enum IOError {
     #[cfg(feature = "gltf-io")]
     #[error("the .gltf file contain missing buffer data")]
     GltfMissingData,
+    #[error("the .vol file contain wrong data size")]
+    VolCorruptData,
     #[cfg(not(target_arch = "wasm32"))]
-    #[error("error while loading a file")]
-    Load(#[from] std::io::Error),
+    #[error("error while loading the file {0}: {1}")]
+    FailedLoading(String, std::io::Error),
+    #[cfg(feature = "reqwest")]
+    #[error("error while loading the url {0}: {1}")]
+    FailedLoadingUrl(String, reqwest::Error),
+    #[cfg(not(feature = "reqwest"))]
+    #[error("error while loading the url {0}: feature 'reqwest' not enabled")]
+    FailedLoadingUrl(String),
     #[error("tried to use {0} which was not loaded")]
     NotLoaded(String),
 }
