@@ -4,9 +4,8 @@ use crate::renderer::*;
 /// Three arrows indicating the three main axes; the x-axis (red), the y-axis (green) and the z-axis (blue).
 /// Used for easily debugging where objects are placed in the 3D world.
 ///
-#[derive(Clone)]
 pub struct Axes {
-    model: Model<ColorMaterial>,
+    model: std::cell::RefCell<Model<ColorMaterial>>,
     aabb_local: AxisAlignedBoundingBox,
     aabb: AxisAlignedBoundingBox,
     transformation: Mat4,
@@ -17,8 +16,8 @@ impl Axes {
     /// Creates a new axes object consisting of three arrows with the given radius and length.
     ///
     pub fn new(context: &Context, radius: f32, length: f32) -> ThreeDResult<Self> {
-        let mut mesh = CPUMesh::arrow(0.9, 0.6, 16);
-        mesh.transform(&Mat4::from_nonuniform_scale(length, radius, radius));
+        let mut mesh = CpuMesh::arrow(0.9, 0.6, 16);
+        mesh.transform(&Mat4::from_nonuniform_scale(length, radius, radius))?;
         let model = Model::new(context, &mesh)?;
         let mut aabb = model.aabb();
         let mut aabb2 = aabb.clone();
@@ -28,51 +27,29 @@ impl Axes {
         aabb3.transform(&Mat4::from_angle_y(degrees(-90.0)));
         aabb.expand_with_aabb(&aabb3);
         Ok(Self {
-            model,
+            model: std::cell::RefCell::new(model),
             aabb: aabb.clone(),
             aabb_local: aabb,
             transformation: Mat4::identity(),
         })
     }
-}
 
-impl Shadable for Axes {
-    fn render_with_material(
-        &self,
-        material: &dyn Material,
-        camera: &Camera,
-        lights: &Lights,
-    ) -> ThreeDResult<()> {
-        let mut model = self.model.clone();
-        model.render_with_material(material, camera, lights)?;
-        model.set_transformation(self.transformation * Mat4::from_angle_z(degrees(90.0)));
-        model.render_with_material(material, camera, lights)?;
-        model.set_transformation(self.transformation * Mat4::from_angle_y(degrees(-90.0)));
-        model.render_with_material(material, camera, lights)
+    ///
+    /// Returns the local to world transformation applied to the axes.
+    ///
+    pub fn transformation(&self) -> Mat4 {
+        self.transformation
     }
 
-    fn render_forward(
-        &self,
-        material: &dyn Material,
-        camera: &Camera,
-        lights: &Lights,
-    ) -> ThreeDResult<()> {
-        self.render_with_material(material, camera, lights)
-    }
-
-    #[allow(deprecated)]
-    fn render_deferred(
-        &self,
-        material: &DeferredPhysicalMaterial,
-        camera: &Camera,
-        viewport: Viewport,
-    ) -> ThreeDResult<()> {
-        let mut model = self.model.clone();
-        model.render_deferred(material, camera, viewport)?;
-        model.set_transformation(self.transformation * Mat4::from_angle_z(degrees(90.0)));
-        model.render_deferred(material, camera, viewport)?;
-        model.set_transformation(self.transformation * Mat4::from_angle_y(degrees(-90.0)));
-        model.render_deferred(material, camera, viewport)
+    ///
+    /// Set the local to world transformation applied to the axes.
+    /// Can be used to visualize a local coordinate system.
+    ///
+    pub fn set_transformation(&mut self, transformation: Mat4) {
+        self.transformation = transformation;
+        let mut aabb = self.aabb_local.clone();
+        aabb.transform(&self.transformation);
+        self.aabb = aabb;
     }
 }
 
@@ -81,47 +58,67 @@ impl Geometry for Axes {
         self.aabb
     }
 
-    fn transformation(&self) -> Mat4 {
-        self.transformation
-    }
-}
-impl GeometryMut for Axes {
-    fn set_transformation(&mut self, transformation: Mat4) {
-        self.transformation = transformation;
-        let mut aabb = self.aabb_local.clone();
-        aabb.transform(&self.transformation);
-        self.aabb = aabb;
+    fn render_with_material(
+        &self,
+        material: &dyn Material,
+        camera: &Camera,
+        lights: &[&dyn Light],
+    ) -> ThreeDResult<()> {
+        self.model
+            .borrow_mut()
+            .set_transformation(self.transformation);
+        self.model
+            .borrow()
+            .render_with_material(&material, camera, lights)?;
+        self.model
+            .borrow_mut()
+            .set_transformation(self.transformation * Mat4::from_angle_z(degrees(90.0)));
+        self.model
+            .borrow()
+            .render_with_material(&material, camera, lights)?;
+        self.model
+            .borrow_mut()
+            .set_transformation(self.transformation * Mat4::from_angle_y(degrees(-90.0)));
+        self.model
+            .borrow()
+            .render_with_material(material, camera, lights)
     }
 }
 
 impl Object for Axes {
-    fn render(&self, camera: &Camera, _lights: &Lights) -> ThreeDResult<()> {
-        let mut model = self.model.clone();
-        model.render_with_material(
+    fn render(&self, camera: &Camera, _lights: &[&dyn Light]) -> ThreeDResult<()> {
+        self.model
+            .borrow_mut()
+            .set_transformation(self.transformation);
+        self.model.borrow().render_with_material(
             &ColorMaterial {
                 color: Color::RED,
                 ..Default::default()
             },
             camera,
-            &Lights::default(),
+            &[],
         )?;
-        model.set_transformation(self.transformation * Mat4::from_angle_z(degrees(90.0)));
-        model.render_with_material(
+        self.model
+            .borrow_mut()
+            .set_transformation(self.transformation * Mat4::from_angle_z(degrees(90.0)));
+        self.model.borrow().render_with_material(
             &ColorMaterial {
                 color: Color::GREEN,
                 ..Default::default()
             },
             camera,
-            &Lights::default(),
+            &[],
         )?;
-        model.set_transformation(self.transformation * Mat4::from_angle_y(degrees(-90.0)));
-        model.render_with_material(
+        self.model
+            .borrow_mut()
+            .set_transformation(self.transformation * Mat4::from_angle_y(degrees(-90.0)));
+        self.model.borrow().render_with_material(
             &ColorMaterial {
                 color: Color::BLUE,
                 ..Default::default()
             },
             camera,
-            &Lights::default(),
+            &[],
         )?;
         Ok(())
     }
