@@ -1,5 +1,8 @@
 use crate::renderer::*;
 
+///
+/// A bounding box object used for visualising an [AxisAlignedBoundingBox].
+///
 pub struct BoundingBox<M: Material> {
     model: InstancedModel<M>,
     aabb: AxisAlignedBoundingBox,
@@ -13,11 +16,26 @@ impl<M: Material> BoundingBox<M> {
         context: &Context,
         aabb: AxisAlignedBoundingBox,
         material: M,
+    ) -> ThreeDResult<Self> {
+        let size = aabb.size();
+        let thickness = 0.02 * size.x.max(size.y).max(size.z);
+
+        Self::new_with_material_and_thickness(context, aabb, material, thickness)
+    }
+
+    ///
+    /// Creates a bounding box object from an axis aligned bounding box with a specified line
+    /// thickness.
+    ///
+    pub fn new_with_material_and_thickness(
+        context: &Context,
+        aabb: AxisAlignedBoundingBox,
+        material: M,
+        thickness: f32,
     ) -> ThreeDResult<Self> {
         let max = aabb.max();
         let min = aabb.min();
         let size = aabb.size();
-        let thickness = 0.02 * size.x.max(size.y).max(size.z);
         let transformations = [
             Mat4::from_translation(min) * Mat4::from_nonuniform_scale(size.x, thickness, thickness),
             Mat4::from_translation(vec3(min.x, max.y, max.z))
@@ -55,60 +73,35 @@ impl<M: Material> BoundingBox<M> {
             context,
             &transformations
                 .iter()
-                .map(|t| ModelInstance {
+                .map(|t| Instance {
                     geometry_transform: *t,
                     ..Default::default()
                 })
                 .collect::<Vec<_>>(),
-            &CPUMesh::cylinder(16),
+            &CpuMesh::cylinder(16),
             material,
         )?;
         Ok(Self { model, aabb })
     }
 }
 
-impl<M: Material> Shadable for BoundingBox<M> {
-    fn render_with_material(
-        &self,
-        material: &dyn Material,
-        camera: &Camera,
-        lights: &Lights,
-    ) -> ThreeDResult<()> {
-        self.model.render_with_material(material, camera, lights)
-    }
-
-    fn render_forward(
-        &self,
-        material: &dyn Material,
-        camera: &Camera,
-        lights: &Lights,
-    ) -> ThreeDResult<()> {
-        self.render_with_material(material, camera, lights)
-    }
-
-    #[allow(deprecated)]
-    fn render_deferred(
-        &self,
-        material: &DeferredPhysicalMaterial,
-        camera: &Camera,
-        viewport: Viewport,
-    ) -> ThreeDResult<()> {
-        self.model.render_deferred(material, camera, viewport)
-    }
-}
-
 impl<M: Material> Geometry for BoundingBox<M> {
     fn aabb(&self) -> AxisAlignedBoundingBox {
         self.aabb
     }
 
-    fn transformation(&self) -> Mat4 {
-        Mat4::identity()
+    fn render_with_material(
+        &self,
+        material: &dyn Material,
+        camera: &Camera,
+        lights: &[&dyn Light],
+    ) -> ThreeDResult<()> {
+        self.model.render_with_material(material, camera, lights)
     }
 }
 
 impl<M: Material> Object for BoundingBox<M> {
-    fn render(&self, camera: &Camera, lights: &Lights) -> ThreeDResult<()> {
+    fn render(&self, camera: &Camera, lights: &[&dyn Light]) -> ThreeDResult<()> {
         self.model.render(camera, lights)
     }
 
