@@ -1,6 +1,8 @@
 use crate::renderer::*;
 
-#[derive(Clone)]
+///
+/// A circle 2D object which can be rendered.
+///
 pub struct Circle<M: Material> {
     context: Context,
     model: Model<M>,
@@ -9,13 +11,16 @@ pub struct Circle<M: Material> {
 }
 
 impl<M: Material> Circle<M> {
+    ///
+    /// Constructs a new circle object with the given material.
+    ///
     pub fn new_with_material(
         context: &Context,
         center: Vec2,
         radius: f32,
         material: M,
     ) -> ThreeDResult<Self> {
-        let mesh = CPUMesh::circle(64);
+        let mesh = CpuMesh::circle(64);
         let mut circle = Self {
             context: context.clone(),
             model: Model::new_with_material(context, &mesh, material)?,
@@ -26,20 +31,24 @@ impl<M: Material> Circle<M> {
         Ok(circle)
     }
 
+    /// Set the radius of the circle.
     pub fn set_radius(&mut self, radius: f32) {
         self.radius = radius;
         self.update();
     }
 
+    /// Get the radius of the circle.
     pub fn radius(&self) -> f32 {
         self.radius
     }
 
+    /// Set the center of the circle.
     pub fn set_center(&mut self, center: Vec2) {
         self.center = center;
         self.update();
     }
 
+    /// Get the center of the circle.
     pub fn center(&self) -> &Vec2 {
         &self.center
     }
@@ -51,28 +60,22 @@ impl<M: Material> Circle<M> {
     }
 }
 
-impl<M: Material> Shadable2D for Circle<M> {
+impl<M: Material> Geometry2D for Circle<M> {
     fn render_with_material(
         &self,
         material: &dyn Material,
         viewport: Viewport,
     ) -> ThreeDResult<()> {
         self.context.camera2d(viewport, |camera2d| {
-            self.model
-                .render_with_material(material, camera2d, &Lights::default())
+            self.model.render_with_material(material, camera2d, &[])
         })
     }
-
-    fn render_forward(&self, material: &dyn Material, viewport: Viewport) -> ThreeDResult<()> {
-        self.render_with_material(material, viewport)
-    }
 }
 
 impl<M: Material> Object2D for Circle<M> {
     fn render(&self, viewport: Viewport) -> ThreeDResult<()> {
-        self.context.camera2d(viewport, |camera2d| {
-            self.model.render(camera2d, &Lights::default())
-        })
+        self.context
+            .camera2d(viewport, |camera2d| self.model.render(camera2d, &[]))
     }
 
     fn is_transparent(&self) -> bool {
