@@ -1,39 +1,46 @@
 use crate::renderer::*;
 
-#[derive(Clone)]
+///
+/// A line 2D object which can be rendered.
+///
 pub struct Line<M: Material> {
     context: Context,
     model: Model<M>,
     pixel0: Vec2,
     pixel1: Vec2,
-    width: f32,
+    thickness: f32,
 }
 
 impl<M: Material> Line<M> {
+    ///
+    /// Constructs a new line object with the given material.
+    ///
     pub fn new_with_material(
         context: &Context,
         pixel0: Vec2,
         pixel1: Vec2,
-        width: f32,
+        thickness: f32,
         material: M,
     ) -> ThreeDResult<Self> {
-        let mut mesh = CPUMesh::square();
-        mesh.transform(&(Mat4::from_scale(0.5) * Mat4::from_translation(vec3(1.0, 0.0, 0.0))));
+        let mut mesh = CpuMesh::square();
+        mesh.transform(&(Mat4::from_scale(0.5) * Mat4::from_translation(vec3(1.0, 0.0, 0.0))))?;
         let mut line = Self {
             context: context.clone(),
             model: Model::new_with_material(context, &mesh, material)?,
             pixel0,
             pixel1,
-            width,
+            thickness,
         };
         line.update();
         Ok(line)
     }
 
+    /// Get one of the end points of the line.
     pub fn end_point0(&self) -> Vec2 {
         self.pixel0
     }
 
+    /// Get one of the end points of the line.
     pub fn end_point1(&self) -> Vec2 {
         self.pixel1
     }
@@ -49,8 +56,9 @@ impl<M: Material> Line<M> {
         self.update();
     }
 
-    pub fn set_width(&mut self, width: f32) {
-        self.width = width;
+    /// Set the line thickness.
+    pub fn set_thickness(&mut self, thickness: f32) {
+        self.thickness = thickness;
         self.update();
     }
 
@@ -64,33 +72,27 @@ impl<M: Material> Line<M> {
         self.model.set_transformation_2d(
             Mat3::from_translation(self.pixel0)
                 * rot
-                * Mat3::from_nonuniform_scale(length, self.width),
+                * Mat3::from_nonuniform_scale(length, self.thickness),
         );
     }
 }
 
-impl<M: Material> Shadable2D for Line<M> {
+impl<M: Material> Geometry2D for Line<M> {
     fn render_with_material(
         &self,
         material: &dyn Material,
         viewport: Viewport,
     ) -> ThreeDResult<()> {
         self.context.camera2d(viewport, |camera2d| {
-            self.model
-                .render_with_material(material, camera2d, &Lights::default())
+            self.model.render_with_material(material, camera2d, &[])
         })
     }
-
-    fn render_forward(&self, material: &dyn Material, viewport: Viewport) -> ThreeDResult<()> {
-        self.render_with_material(material, viewport)
-    }
 }
 
 impl<M: Material> Object2D for Line<M> {
     fn render(&self, viewport: Viewport) -> ThreeDResult<()> {
-        self.context.camera2d(viewport, |camera2d| {
-            self.model.render(camera2d, &Lights::default())
-        })
+        self.context
+            .camera2d(viewport, |camera2d| self.model.render(camera2d, &[]))
     }
 
     fn is_transparent(&self) -> bool {
