@@ -43,13 +43,16 @@ impl Window {
         }
 
         let windowed_context = unsafe { wc?.make_current().unwrap() };
-        let context = crate::context::GLContext::load_with(|s| {
-            windowed_context.get_proc_address(s) as *const std::os::raw::c_void
-        });
+
+        let context = unsafe {
+            crate::context::Context::from_loader_function(|s| {
+                windowed_context.get_proc_address(s) as *const _
+            })
+        };
         Ok(Window {
             windowed_context,
             event_loop,
-            gl: crate::core::Context::from_gl_context(context),
+            gl: crate::core::Context::from_gl_context(std::rc::Rc::new(context))?,
         })
     }
 
@@ -103,6 +106,7 @@ impl Window {
         let mut modifiers = Modifiers::default();
         let mut first_frame = true;
         let mut mouse_pressed = None;
+        #[cfg(feature = "image-io")]
         let context = self.gl.clone();
         self.event_loop.run(move |event, _, control_flow| {
             match event {
@@ -140,7 +144,16 @@ impl Window {
                     };
                     first_frame = false;
                     events.clear();
-                    let frame_output = callback(frame_input);
+                    let mut frame_output = callback(frame_input);
+                    if let Ok(v) = std::env::var("THREE_D_SCREENSHOT") {
+                        frame_output.screenshot = Some(v.into());
+                    }
+                    if let Ok(v) = std::env::var("THREE_D_EXIT") {
+                        if v.parse::<f64>().unwrap() < accumulated_time {
+                            frame_output.exit = true;
+                        }
+                    }
+
                     if frame_output.exit {
                         *control_flow = ControlFlow::Exit;
                     } else {
