@@ -13,23 +13,25 @@ impl Context {
     pub fn new() -> ThreeDResult<Self> {
         let cb = ContextBuilder::new();
         let (headless_context, _el) = build_context(cb).unwrap();
-        let current_context = unsafe { headless_context.make_current().unwrap() };
-        Self::from_gl_context(std::rc::Rc::new(unsafe {
+        let headless_context = unsafe { headless_context.make_current().unwrap() };
+        let mut c = Self::from_gl_context(std::rc::Rc::new(unsafe {
             crate::context::Context::from_loader_function(|s| {
-                current_context.get_proc_address(s) as *const _
+                headless_context.get_proc_address(s) as *const _
             })
-        }))
+        }))?;
+        c.glutin_context = Some(std::rc::Rc::new(headless_context));
+        Ok(c)
     }
 }
 
-#[cfg(target_os = "linux")]
+/*#[cfg(target_os = "linux")]
 fn build_context_surfaceless<T1: ContextCurrentState>(
     cb: ContextBuilder<T1>,
     el: &EventLoop<()>,
 ) -> Result<glutin::Context<NotCurrent>, CreationError> {
     use glutin::platform::unix::HeadlessContextExt;
     cb.build_surfaceless(&el)
-}
+}*/
 
 fn build_context_headless<T1: ContextCurrentState>(
     cb: ContextBuilder<T1>,
@@ -51,7 +53,7 @@ fn build_context_osmesa<T1: ContextCurrentState>(
 #[cfg(target_os = "linux")]
 fn build_context<T1: ContextCurrentState>(
     cb: ContextBuilder<T1>,
-) -> Result<(glutin::Context<NotCurrent>, EventLoop<()>), [CreationError; 3]> {
+) -> Result<(glutin::Context<NotCurrent>, EventLoop<()>), [CreationError; 2]> {
     // On unix operating systems, you should always try for surfaceless first,
     // and if that does not work, headless (pbuffers), and if that too fails,
     // finally osmesa.
@@ -62,11 +64,11 @@ fn build_context<T1: ContextCurrentState>(
     use glutin::platform::unix::EventLoopExtUnix;
     let el = EventLoopExtUnix::new_any_thread();
 
-    println!("Trying surfaceless");
+    /*println!("Trying surfaceless");
     let err1 = match build_context_surfaceless(cb.clone(), &el) {
         Ok(ctx) => return Ok((ctx, el)),
         Err(err) => err,
-    };
+    };*/
 
     println!("Trying headless");
     let err2 = match build_context_headless(cb.clone(), &el) {
@@ -80,7 +82,7 @@ fn build_context<T1: ContextCurrentState>(
         Err(err) => err,
     };
 
-    Err([err1, err2, err3])
+    Err([err2, err3])
 }
 
 #[cfg(not(target_os = "linux"))]
